--//2205EN_02_CodeFile.sql: Chapter 2 Code --//

--Code snippet #1: CREATE DATABASE statement example 1 
--Creating a database based on model database
--The following CREATE DATABASE script creates CH02_01 database using default parameters from model database
USE [master];
GO

CREATE DATABASE [CH02_01];
GO

--Code snippet #2: CREATE DATABASE statement example 2
--The following CREATE DATABASE script creates CH02_02 database by explicitly specifying data and transaction log file filespecs properties
USE [master];
GO

CREATE DATABASE CH02_02 ON PRIMARY
(NAME='CH02_02_Data', FILENAME = 'C:\SQLDATA\CH02_02.mdf', SIZE=10MB, MAXSIZE=20, FILEGROWTH=10%)
    LOG ON
(NAME='CH02_02_log', FILENAME = 'C:\SQLLog\CH02_02_log.ldf',
SIZE=10MB, MAXSIZE=200, FILEGROWTH=20%);
GO

--Code snippet #3: CREATE DATABASE statement example 3
--The following CREATE DATABASE statement creates CH02_03 database on following two filegroups: 
-- + The primary filegroup contains the following two database files: CH02_03DAT01 and CH02_02DAT02
-- + The user-defined filegroup called CH02_FG1 that only contains the database file: CH02_03DAT03
USE [master];
GO

CREATE DATABASE [CH02_03] 
CONTAINMENT = NONE
    ON PRIMARY 
(NAME = N'CH02_03DAT01 ', FILENAME = N'C:\Program Files\Microsoft SQL Server\MSSQL12.MSSQLSERVER\MSSQL\DATA\CH02_03DAT01 .mdf', SIZE = 524288KB, FILEGROWTH = 102400KB),
(NAME = N'CH02_03DAT02', FILENAME = N'C:\Program Files\Microsoft SQL Server\MSSQL12.MSSQLSERVER\MSSQL\DATA\CH02_03DAT02.ndf', SIZE = 524288KB, FILEGROWTH = 102400KB),
    FILEGROUP [CH02_FG1] 
(NAME = N'CH02_03DAT03', FILENAME = N'C:\Program Files\Microsoft SQL Server\MSSQL12.MSSQLSERVER\MSSQL\DATA\CH02_03DAT03.ndf', SIZE = 262144KB, FILEGROWTH = 102400KB)
    LOG ON 
(NAME = N'CH02_03_log', FILENAME = N'C:\Program Files\Microsoft SQL Server\MSSQL12.MSSQLSERVER\MSSQL\DATA\CH02_03_log.ldf', SIZE = 262144KB, FILEGROWTH = 102400KB)
GO

--Code snippet #4: ALTER DATABASE statement example 1
--The following example uses ALTER DATABASE to add secondary data file to the CH2_03 database user-defined filegroup "CH02_FG1"
USE [master];
GO

ALTER DATABASE [CH02_03] ADD FILE (NAME = N'CH02_03DAT04', FILENAME = N'C:\Program Files\Microsoft SQL Server\MSSQL12.MSSQLSERVER\MSSQL\DATA\CH02_03DAT04.ndf', SIZE = 524288KB, FILEGROWTH = 102400KB) TO FILEGROUP [CH02_FG1];
GO

--Code snippet #5: ALTER DATABASE statement example 2
--The following ALTER DATABASE statement sets the CH02_03 database recovery model FULL
USE [master];
GO

ALTER DATABASE [CH02_03] SET RECOVERY FULL WITH NO_WAIT;
GO

--Code snippet #6: DROP DATABASE statement example
--The following script uses DROP DATABASE statement to drop CH02_01 database  from SQL Server
USE [master];
GO

DROP DATABASE CH02_01;
GO

--Code snippet #6: Using CREATE TABLE statement example 1
--The following T-SQL code uses CREATE TABLE statement, to create Book_Info table inside CH02_03 database
USE [CH02_03];
GO
CREATE TABLE Book_Info
    (
      Book_ID SMALLINT ,
      Book_Name VARCHAR(20) ,
      Description VARCHAR(30) ,
      Price [SMALLMONEY] ,
      Author_ID [int]
    )
ON  CH02_FG1;
GO
-- Note: author_type is a user_defined data type. For more information about user-defined data types, refer to chapter 1 section "Improtance of choosing appropriate data type"

--Code snippet #6: Using CREATE TABLE statement example 2
--// Steps to create memory-optimized tables //--
--To create memory-optimized tables, you must first create an MEMORY_OPTIMIZED_DATA filegroup within a database. 
--For example, to create a MEMORY_OPTIMIZED_DATA filegroup in CH02_03 database, you enter and execute the following T-SQL code
USE [master];
GO

ALTER DATABASE [CH02_03]
ADD FILEGROUP [CH02_FGMO]
CONTAINS MEMORY_OPTIMIZED_DATA;
GO

--Next, run the following code to add database file to the CH02_03 database memory-optimized file group (CH02_FGMO)
USE [master];
GO

ALTER DATABASE [CH02_03]
ADD FILE (NAME = 'CH02_03_MemoryOptimized', FILENAME = 'C:\SQLData\CH02_03_MO.ndf')
TO FILEGROUP CH02_FGMO
GO

--Finally, enter and execute the following T-SQL code, to create memory-optimized version of Book_Info table within a CH02_03 database
USE [CH02_03];
GO

CREATE TABLE Book_Info_MO
    (
      Book_ID SMALLINT NOT NULL,
      Book_Name VARCHAR(20),
      Description VARCHAR(30),
      Price [SMALLMONEY],
      Author_ID [int],
	  CONSTRAINT [PK_Book_Info_ID] PRIMARY KEY
	  NONCLUSTERED HASH (Book_ID) WITH (BUCKET_COUNT = 2000))
	  WITH (MEMORY_OPTIMIZED = ON, DURABILITY = SCHEMA_AND_DATA);
GO

--Code snippet #7: ALTER TABLE statement example
--The following T-SQL code uses ALTER TABLE statement add a Topic_ID column to Book_Info table
USE [CH02_03];
GO

ALTER TABLE [Book_Info]
ADD [Topic_ID] INT NOT NULL;
GO

--Code snippet #8: DROP TABLE statement example
--The following DROP TABLE statement deletes Book_Info table from CH02_03 database
USE [CH02_03];
GO
DROP TABLE Book_Info_MO;
GO

--The following two examples assumes that user Bob exists on the SQL Server. 
--Code snippet #9: GRANT statement example
--The following GRANT statement grants Bob the SELECT permission WITH GRANT OPTION on the Book_Info table
USE [CH02_03];
GO

GRANT SELECT ON [dbo].[Book_Info] TO [Bob]
WITH GRANT OPTION;
GO

--Code snippet #10: DENY statement example
--The following T-SQL code uses DENY statement, to deny Bob the UPDATE permission on the Book_Info table
USE [CH02_03];
GO

DENY UPDATE ON [dbo].[Book_Info] TO [Bob];
GO
