--// 2205EN_03_CodeFile.sql: Chapter 3 Code //-- 

--Change AdventureWorks2012 database compatibility level to SQL Server 2014
USE [master];
GO

ALTER DATABASE [AdventureWorks2012] SET COMPATIBILITY_LEVEL = 120;
GO

USE [AdventureWorks2012];
GO

--Code snippet #1: SELECT statement example 1 
--The following query returns all columns from the Sales.Currency table.
SELECT  *
FROM    [Sales].[Currency];

--Code snippet #2: SELECT statement example 2
--The following query only displays the CurrencyCode and Name column from the Sales.Currency table
SELECT  [CurrencyCode] ,
        [Name]
FROM    [Sales].[Currency];

--Code snippet #3: SELECT statement with WHERE clause example
--Find all the currency codes that begin with the character "A". 
SELECT  [CurrencyCode] ,
        [Name]
FROM    [Sales].[Currency]
WHERE   [CurrencyCode] LIKE N'A%';

--// Using T-SQL function in your queries //--
--Code snippet #4: Using Aggregate functions in SELECT statement example 1 
--The query uses AVG function, to calculate the average unit price for all orders
SELECT  AVG(OrderQty * UnitPrice) AS [Avg]
FROM    [Sales].[SalesOrderDetail];

--Code snippet #5: Using Aggregate functions in SELECT statement example 2
--The query uses COUNT function, to count the number of orders where the ProductID is 777
SELECT  COUNT(*)
FROM    [Sales].[SalesOrderDetail]
WHERE   ProductID = 777;

--Code snippet #6: Using Configuration functions in SELECT statement example
--The following query uses @@SERVERNAME and @@VERSION functions, to return the SQL Server name and version information
SELECT  @@SERVERNAME ,
        @@VERSION;

--Code snippet #7: Date and time function in SELECT statement example 1
--The following query uses GETDATE and CURRENT_TIMESTAMP functions, to return the current system date and time
SELECT  GETDATE() ,
        CURRENT_TIMESTAMP;

--Code snippet #8: Date and time function in SELECT statement example 2
--The following query using DATEDIFF function, to find a number of days since 1st January 2014
SELECT  DATEDIFF(DAY, '01-01-2014', CURRENT_TIMESTAMP);

--Code snippet #9: Query using mathematical function 
--The following query uses ROUND function, to round off values of AvergateRate and EndOfDayRate columns of Sales.CurrencyRate table 
--to one decimal place
SELECT  [FromCurrencyCode] ,
        [ToCurrencyCode] ,
        ROUND([AverageRate], 1) ,
        ROUND([EndOfDayRate], 1)
FROM    [Sales].[CurrencyRate];

--Code snippet #10: Metadata function in SELECT statement example
--The following query using OBJECT_NAME and STATS_DATE function, to determine date when the statistics were last updated for each 
--statistics object that exists for the tables, indexes, and indexed views in the database
SELECT  OBJECT_NAME(object_id) ,
        [name] AS [StatisticName] ,
        STATS_DATE([object_id], [stats_id])
FROM    sys.stats;

--Code snippet #11: String function in SELECT statement example 
--The following example uses CONCAT function to concatenate FirstName, MiddleName, and LastName columns of Person.Person table
SELECT  CONCAT([FirstName] + SPACE(1), [MiddleName] + SPACE(1), [LastName])
FROM    [Person].[Person];

--Code snippet #12:  UNION operator example
--The following query uses UNION operator, to return the list of all products associated with purchase order and all products 
--associated with sales order. However, it removes duplicate records returned in result
SELECT  [ProductID] ,
        [UnitPrice] ,
        [OrderQty]
FROM    [Purchasing].[PurchaseOrderDetail]
UNION
SELECT  [ProductID] ,
        [UnitPrice] ,
        [OrderQty]
FROM    [Sales].[SalesOrderDetail];

--Code snippet #13: UNION ALL operator example 
--The following query uses UNION ALL operator, to return the list of all products associated with purchase order and all products
--associated with sales order. However, it keeps duplicate records returned in the result set
SELECT  [ProductID] ,
        [UnitPrice] ,
        [OrderQty]
FROM    [Purchasing].[PurchaseOrderDetail]
UNION ALL
SELECT  [ProductID] ,
        [UnitPrice] ,
        [OrderQty]
FROM    [Sales].[SalesOrderDetail];

--Code snippet #14: EXCEPT operator example 
--The following query uses EXCEPT operator, to return only distinct ProductID, UnitPrice and OrderQty of those products associated
--with the purchase orders that do not have an associated sales order
SELECT  [ProductID] ,
        [UnitPrice] ,
        [OrderQty]
FROM    [Purchasing].[PurchaseOrderDetail]
EXCEPT
SELECT  [ProductID] ,
        [UnitPrice] ,
        [OrderQty]
FROM    [Sales].[SalesOrderDetail];

--Code snippet #15: INTERSECT operator example 
--The following query uses INTERSECT operator, to find out which CurrencyCode exist in both Sales.Currency table and  the 
--Sales.CurrencyRate table, get the distinct list of CurrencyCode back, you would run the following
SELECT  [CurrencyCode]
FROM    [Sales].[Currency]
INTERSECT
SELECT  [ToCurrencyCode]
FROM    [Sales].[CurrencyRate];

--Code snippet #16: INNER JOIN example 
--The following query uses INNER JOIN, to retrieve all the employees name, title and current department information
SELECT  p.[Title] ,
        p.[FirstName] + SPACE(1) + p.[MiddleName] + SPACE(1) + p.[LastName] AS [FullName] ,
        e.[JobTitle] ,
        d.[Name] AS [Department] ,
        d.[GroupName] ,
        dhist.[StartDate]
FROM    [HumanResources].[Employee] e
        INNER JOIN [HumanResources].[EmployeeDepartmentHistory] dhist ON e.[BusinessEntityID] = dhist.[BusinessEntityID]
                                                              AND dhist.[EndDate] IS NULL
        INNER JOIN [Person].[Person] p ON p.[BusinessEntityID] = e.[BusinessEntityID]
        INNER JOIN [HumanResources].[Department] d ON dhist.[DepartmentID] = d.[DepartmentID];

--Code snippet #17: LEFT OUTER JOIN example 
--The following query uses LEFT OUTER JOIN, to retrieve the employee's name and title, regardless of whether they have 
--phone, email and additional contact information
SELECT  p.[Title] ,
        p.[FirstName] + SPACE(1) + p.[MiddleName] + SPACE(1) + p.[LastName] AS [FullName] ,
        pp.[PhoneNumber] ,
        pt.[Name] AS [PhoneNumberType] ,
        ea.[EmailAddress] ,
        p.[AdditionalContactInfo]
FROM    [HumanResources].[Employee] e
        INNER JOIN [Person].[Person] p ON p.[BusinessEntityID] = e.[BusinessEntityID]
        LEFT OUTER JOIN [Person].[EmailAddress] ea ON p.[BusinessEntityID] = ea.[BusinessEntityID]
        LEFT OUTER JOIN [Person].[PersonPhone] pp ON pp.[BusinessEntityID] = p.[BusinessEntityID]
        LEFT OUTER JOIN [Person].[PhoneNumberType] pt ON pp.[PhoneNumberTypeID] = pt.[PhoneNumberTypeID];

--Code snippet #18: RIGHT OUTER JOIN example 
--The following query uses RIGHT OUTER JOIN to return the list of all products, regardless of whether special
--discount associated with the product
SELECT  p.[ProductID] ,
        p.[Name] ,
        so.[SpecialOfferID] ,
        so.[Description] ,
        so.[DiscountPct] ,
        so.[Type] ,
        so.[Category] ,
        so.[StartDate] ,
        so.[EndDate] ,
        so.[MinQty] ,
        so.[MaxQty]
FROM    [Sales].[SpecialOfferProduct] sop
        RIGHT OUTER JOIN [Sales].[SpecialOffer] so ON so.[SpecialOfferID] = sop.[SpecialOfferID]
        RIGHT OUTER JOIN [Production].[Product] p ON p.[ProductID] = sop.[ProductID];

--Code snippet #19: CROSS JOIN example
SELECT  c.* ,
        crc.*
FROM    [Sales].[Currency] c
        CROSS JOIN [Sales].[CountryRegionCurrency] crc;

--Code snippet #20: Subquery example 1 
--The following query contains subquery, returns the names of all employees who have a valid email address
SELECT  [Title] ,
        [FirstName] ,
        [MiddleName] ,
        [LastName]
FROM    [Person].[Person] p
WHERE   EXISTS ( SELECT *
                 FROM   [Person].[EmailAddress] e
                 WHERE  p.[BusinessEntityID] = e.[BusinessEntityID] );

--Code snippet #21: Subquery example 2 
--The following query uses subquery, to return a list of all customers who live in a territory that is not 
--covered by any sales person
SELECT  *
FROM    [Sales].[Customer]
WHERE   [TerritoryID] <> ANY ( SELECT   [TerritoryID]
                               FROM     [Sales].[SalesPerson] );

--Code snippet #22: Subquery example 3
--The following query finds CustomerID and AccountNumbers of all customers who lives in Europe
SELECT  [CustomerID] ,
        [AccountNumber]
FROM    [Sales].[Customer]
WHERE   [TerritoryID] IN ( SELECT   [TerritoryID]
                           FROM     [Sales].[SalesTerritory]
                           WHERE    [Group] = 'Europe' );

--Code snippet #23: CTE example
WITH    cteSalesPerson ( [SalesPersonID], [FullName], [TerritoryName], [SalesQuota], [Bonus], [CommissionPct], [SalesYTD], [SalesLastYear] )
          AS ( SELECT   sp1.[BusinessEntityID] ,
                        sp2.[FirstName] + SPACE(1) + sp2.[LastName] ,
                        st.[Name] ,
                        sp1.[SalesQuota] ,
                        sp1.[Bonus] ,
                        sp1.[CommissionPct] ,
                        sp1.[SalesYTD] ,
                        sp1.[SalesLastYear]
               FROM     [Sales].[SalesPerson] sp1
                        INNER JOIN [Sales].[vSalesPerson] sp2 ON sp2.[BusinessEntityID] = sp1.[BusinessEntityID]
                        INNER JOIN [Sales].[SalesTerritory] st ON st.[TerritoryID] = sp1.[TerritoryID]
               WHERE    sp1.[TerritoryID] IS NOT NULL
             )
    SELECT  *
    FROM    cteSalesPerson;

--Code snippet #24: ORDER BY clause example 
--The following query uses ORDER BY clause, to display sales representative sales related information, ordered 
--by FirstName and LastName
SELECT  *
FROM    [Sales].[vSalesPerson]
ORDER BY [FirstName] ASC ,
        [LastName] ASC;

--Code snippet #25: GROUP BY clause example 1
--The following query uses GROUP BY clause, to count the number of customers in each territory
SELECT  st.[Name] ,
        COUNT(c.[CustomerID]) [TotalCustomer]
FROM    [Sales].[Customer] c
        INNER JOIN [Sales].[SalesTerritory] st ON st.[TerritoryID] = c.[TerritoryID]
GROUP BY st.[Name];

--Code snippet #26: GROUP BY clause example 2
--The following query uses GROUP BY clause with HAVING clause, to only return those territories in which the 
--number of customers is less than 1000
SELECT  st.[Name] ,
        COUNT(c.[CustomerID]) [TotalCustomer]
FROM    [Sales].[Customer] c
        INNER JOIN [Sales].[SalesTerritory] st ON st.[TerritoryID] = c.[TerritoryID]
GROUP BY st.[Name]
HAVING  ( COUNT(c.[CustomerID]) < 1000 );

--Code snippet #27: TOP clause example, to return the list of 10 most expensive products from Production.Product table
SELECT TOP 10
        [ProductID] ,
        [Name] ,
        [ProductNumber] ,
        [ListPrice]
FROM    [Production].[Product]
ORDER BY [ListPrice] DESC;

--Code snippet #28: DISTINCT clause example, to return a distinct list of product names from Production.Product table
SELECT DISTINCT
        [Name]
FROM    [Production].[Product];

--PIVOT data
--Code snippet #29: Pivoting data example
--For example, examine the out of the following query, which returns the list of employees in each group. 
SELECT  p.[BusinessEntityID] ,
        d.[GroupName]
FROM    [HumanResources].[Employee] e
        INNER JOIN [HumanResources].[EmployeeDepartmentHistory] dhist ON e.[BusinessEntityID] = dhist.[BusinessEntityID]
                                                              AND dhist.[EndDate] IS NULL
        INNER JOIN [Person].[Person] p ON p.[BusinessEntityID] = e.[BusinessEntityID]
        INNER JOIN [HumanResources].[Department] d ON dhist.[DepartmentID] = d.[DepartmentID]; 


-- Now, suppose you want to write a query to determine the total number of employees in each group. 
--To accomplish this, you pivot the group column values into columns, along with the count of employees in each group
SELECT  [Executive General and Administration] ,
        [Inventory Management] ,
        [Manufacturing] ,
        [Quality Assurance] ,
        [Research and Development] ,
        [Sales and Marketing]
FROM    ( SELECT    p.BusinessEntityID ,
                    d.[GroupName]
          FROM      [HumanResources].[Employee] e
                    INNER JOIN [HumanResources].[EmployeeDepartmentHistory] dhist ON e.[BusinessEntityID] = dhist.[BusinessEntityID]
                                                              AND dhist.[EndDate] IS NULL
                    INNER JOIN [Person].[Person] p ON p.[BusinessEntityID] = e.[BusinessEntityID]
                    INNER JOIN [HumanResources].[Department] d ON dhist.[DepartmentID] = d.[DepartmentID]
        ) AS a PIVOT
( COUNT(a.[BusinessEntityID]) FOR [GroupName] IN ( [Executive General and Administration],
                                                   [Inventory Management],
                                                   [Manufacturing],
                                                   [Quality Assurance],
                                                   [Research and Development],
                                                   [Sales and Marketing] ) ) AS b;


--// Transact-SQL analytic window functions examples --//
--Code snippet #30: Ranking function (ROW_NUMBER, RANK, DENSE_RANK and NTILE) example
--The following query demonstrates the use of ranking functions based on the SalesQuota column
SELECT  [LastName] ,
        [FirstName] ,
        [SalesQuota] ,
        ROW_NUMBER() OVER ( ORDER BY [SalesQuota] ) [ROW_NUMBER] ,
        RANK() OVER ( ORDER BY [SalesQuota] ) [RANK] ,
        DENSE_RANK() OVER ( ORDER BY [SalesQuota] ) [DENSE_RANK] ,
        NTILE(10) OVER ( ORDER BY [SalesQuota] ) AS [NTILE]
FROM    [Sales].[vSalesPerson];

--Code snippet #31: Using PARTITION BY clause with ranking function (ROW_NUMBER, RANK, DENSE_RANK and NTILE) example
--The following T-SQL query to partition the result set by the CountryRegionName column
SELECT  [LastName] ,
        [FirstName] ,
        [SalesQuota] ,
        [CountryRegionName] ,
        ROW_NUMBER() OVER ( PARTITION BY [CountryRegionName] ORDER BY [SalesQuota] ) [ROW_NUMBER] ,
        RANK() OVER ( PARTITION BY [CountryRegionName] ORDER BY [SalesQuota] ) [RANK] ,
        DENSE_RANK() OVER ( PARTITION BY [CountryRegionName] ORDER BY [SalesQuota] ) [DENSE_RANK] ,
        NTILE(4) OVER ( PARTITION BY [CountryRegionName] ORDER BY [SalesQuota] ) AS [NTILE]
FROM    [Sales].[vSalesPerson];

--Code snippet #32: PERCENT_RANK function example
--The following computes the rank of the salesperson's sales quota within a country as a percentage. 
--The PARTITION BY clause is specified for partitioning of the rows in the result set by country region name, 
--and the ORDER BY clause sorts the rows in each partition
SELECT  [LastName] ,
        [FirstName] ,
        [SalesQuota] ,
        [CountryRegionName] ,
        PERCENT_RANK() OVER ( PARTITION BY [CountryRegionName] ORDER BY [SalesQuota] ) [PERCENT_RANK]
FROM    [Sales].[vSalesPerson]
WHERE   [SalesQuota] IS NOT NULL;

--Code snippet #33: CUME_DIST function example
--The following query uses the CUME_DIST function to calculate the sales quota percentile for each salesperson 
--within a particular country. The value returned by the function CUME_DIST represents the percentage of sales people 
--who have a sales quota less than or equal to the current sales person in the same country
SELECT  [LastName] ,
        [FirstName] ,
        [SalesQuota] ,
        [CountryRegionName] ,
        CUME_DIST() OVER ( PARTITION BY [CountryRegionName] ORDER BY [SalesQuota] ) [CUME_DIST]
FROM    [Sales].[vSalesPerson]
WHERE   [SalesQuota] IS NOT NULL;

--Code snippet #34: PERCENTILE_CONT and PERCENTILE_DESC functions example
--The following T-SQL query uses PERCENTILE_CONT and PERCENTILE_DISC, to find the median employee salary in each business entity
SELECT TOP 15
        [BusinessEntityID] ,
        [Rate] ,
        PERCENTILE_CONT(0.5) WITHIN GROUP ( ORDER BY [Rate] ) OVER ( PARTITION BY [BusinessEntityID] ) [PERCENTILE_CONT] ,
        PERCENTILE_DISC(0.5) WITHIN GROUP ( ORDER BY [Rate] ) OVER ( PARTITION BY [BusinessEntityID] ) [PERCENTILE_DISC]
FROM    [HumanResources].[EmployeePayHistory];

--Code snippet #35: LEAD and LAG functions example
--The following example uses the LEAD and LAG functions to compare year-to-date sales among AdventureWorks employees partitioned by sales territory
SELECT  [FirstName] ,
        [TerritoryGroup] ,
        [SalesYTD] ,
        LEAD([SalesYTD]) OVER ( PARTITION BY [TerritoryGroup] ORDER BY [SalesYTD] ) [Next_Lower_SalesYTD(LEAD)] ,
        LAG([SalesYTD]) OVER ( PARTITION BY [TerritoryGroup] ORDER BY [SalesYTD] ) [Prev_Higer_SalesYTD(LAG)]
FROM    [Sales].[vSalesPerson]
WHERE   [TerritoryGroup] IS NOT NULL;

--Code snippet #36: FIRST_VALUE and LAST_VALUE functions example
--The following query uses FIRST_VALUE and LAST_VALUE functions to return highest and lowest year-to-date sales figures for each sales territory
SELECT DISTINCT
        [TerritoryGroup] ,
        FIRST_VALUE([SalesYTD]) OVER ( PARTITION BY [TerritoryGroup] ORDER BY [SalesYTD]
ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW ) AS [Highest_SalesYTD (FIRST_VALUE)] ,
        LAST_VALUE([SalesYTD]) OVER ( PARTITION BY [TerritoryGroup] ORDER BY [SalesYTD] 
RANGE BETWEEN CURRENT ROW AND UNBOUNDED FOLLOWING ) AS [Lowest_SalesYTD (FIRST_VALUE)]
FROM    [Sales].[vSalesPerson]
WHERE   [TerritoryGroup] IS NOT NULL;