--// 2205EN_04_CodeFile.sql: Chapter 4 Code //-- 


USE [AdventureWorks2012];
GO

-- Create table dbo.CustomProducts
IF OBJECT_ID(N'dbo.CustomProducts', N'U') IS NOT NULL
    DROP TABLE [dbo].[CustomProducts];
GO
CREATE TABLE [dbo].[CustomProducts]
    (
      [ProductID] [int] IDENTITY(1, 1)
                        NOT NULL ,
      [ProductName] [varchar](50) NULL
                                  DEFAULT ( 'Anonymous' ) ,
      [ProductCategory] [varchar](50) NULL
                                      DEFAULT ( 'Anonymous' ) ,
      [ListPrice] [money] NOT NULL
                          DEFAULT ( 1.0 ) ,
      [ListPriceCurrency] VARCHAR(30) DEFAULT ( 'US Dollar' ) ,
      [SellStartDate] [datetime] NOT NULL
                                 DEFAULT CURRENT_TIMESTAMP ,
      [SellEndDate] [datetime] NULL
    )
ON  [PRIMARY];
GO

--Code snippet #1: INSERT statement example 
--The following INSERT statement adds a single row to the dbo.CustomProducts table
INSERT  INTO [dbo].[CustomProducts]
        ( [ProductName] ,
          [ProductCategory] ,
          [ListPrice] ,
          [ListPriceCurrency] ,
          [SellStartDate] ,
          [SellEndDate]
        )
VALUES  ( N'iPhone 5s' ,
          N'Gadget' ,
          25 ,
          N'GB' ,
          CURRENT_TIMESTAMP ,
          DEFAULT
        );
GO

--Code snippet #2: INSERT statement to add multiple rows example
--The following INSERT statement adds three rows to the dbo.CustomProducts table
INSERT  INTO [dbo].[CustomProducts]
        ( [ProductName], [ProductCategory], [ListPrice], [ListPriceCurrency],
          [SellStartDate], [SellEndDate] )
VALUES  ( N'Samsung Galaxy S5', N'Gadget', 426, N'GB', CURRENT_TIMESTAMP,
          DEFAULT )
	   ,
        ( N'HTC One (M8)', N'Gadget', 609, N'USD', CURRENT_TIMESTAMP, DEFAULT ),
        ( N'Nokia Lumia 1520', N'Gadget', 529, N'USD', CURRENT_TIMESTAMP,
          DEFAULT );
GO

--Code snippet #3: INSERT with SELECT statement example
--The following INSERT statement uses the SELECT statement to insert rows 
--into the dbo.CustomProducts table
INSERT  INTO [dbo].[CustomProducts]
        ( [ProductName] ,
          [ProductCategory] ,
          [ListPrice] ,
          [ListPriceCurrency] ,
          [SellStartDate] ,
          [SellEndDate]
        )
        SELECT  p.[Name] ,
                pc.[Name] ,
                [ListPrice] ,
                N'USD' ,
                [SellStartDate] ,
                [SellEndDate]
        FROM    [Production].[Product] p
                INNER JOIN [Production].[ProductCategory] pc ON p.[ProductSubcategoryID] = PC.ProductCategoryID;
GO

--Code snippet #4: INSERT with EXEC statement example
--The following INSERT statement inserts the xp_msver stored procedure 
--output into temporary table
IF OBJECT_ID(N'Tempdb..#xp_msver') IS NOT NULL
DROP TABLE #xp_msver
GO

CREATE TABLE #xp_msver
    (
      [idx] [int] NULL ,
      [c_name] [varchar](100) NULL ,
      [int_val] [float] NULL ,
      [c_val] [varchar](128) NULL
    )
 
INSERT  INTO #xp_msver
        EXEC ( '[master]..[xp_msver]'
            );
GO

--Code snippet #5: Explicitly inserting data to identity column example
--The following script inserts the data into ProductID column of dbo.CustomProducts table
SET IDENTITY_INSERT [dbo].[CustomProducts] ON;

INSERT  INTO [dbo].[CustomProducts]
        ( [ProductID] ,
          [ProductName] ,
          [ProductCategory] ,
          [ListPrice] ,
          [ListPriceCurrency] ,
          [SellStartDate] ,
          [SellEndDate]
        )
VALUES  ( 110 ,
          N'Samsung Galaxy S4' ,
          N'Gadget' ,
          200 ,
          N'GB' ,
          CURRENT_TIMESTAMP ,
          DEFAULT
        );

SET IDENTITY_INSERT [dbo].[CustomProducts] OFF;
GO

--Code snippet #6: UPDATE statement example 
--The following UPDATE statement updates the list price of the single 
--product in Production.Product table
UPDATE  [Production].[Product]
SET     [ListPrice] = 1670
WHERE   [ProductID] = 13;
GO

--Code snippet #7: UPDATE statement to update multiple rows example 
--The following UPDATE statement updates the list price of the multiple
--products in Production.Product table
UPDATE  [Production].[Product]
SET     [ListPrice] = [ListPrice] + ( [ListPrice] * 0.25 )
WHERE   [ListPrice] < 1000;
GO

UPDATE  [Production].[Product]
SET     [ListPrice] = 200.0
WHERE   [ListPrice] = 0.0;
GO

--Code snippet #8: DELETE statement example 
--The following DELETE statement deletes all rows from the 
--dbo.CustomProducts table in which the ProductCategory column value is Components
DELETE  FROM [dbo].[CustomProducts]
WHERE   [ProductCategory] = N'Components';
GO

--Code snippet #9: DELETE statement to delete all table rows example 
--The following DELETE statement deletes all rows from the dbo.CustomProducts table 
DELETE  FROM [dbo].[CustomProducts];
GO

--Code snippet #10: MERGE statement example
--Prepration code
IF OBJECT_ID(N'dbo.Spices', N'U') IS NOT NULL
    DROP TABLE [dbo].[Spices];
GO
CREATE TABLE [dbo].[Spices]
    (
      [SpiceID] [int] PRIMARY KEY ,
      [SpiceMixName] [nvarchar](64) ,
      [Supplier] [nvarchar](50)
    );

IF OBJECT_ID(N'dbo.Spices_New', N'U') IS NOT NULL
    DROP TABLE [dbo].[Spices_New];
GO
CREATE TABLE [dbo].[Spices_New]
    (
      [SpiceID] [int] PRIMARY KEY ,
      [SpiceMixName] [nvarchar](64) ,
      [Supplier] [nvarchar](50)
    );

INSERT  INTO [dbo].[Spices]
VALUES  ( 1, N'Five-spice powder', N'Go it!' )
,       ( 2, N'Curry powder', N'East-end Spices' )
,       ( 3, N'Garam masala', N'All Spices' )
,       ( 4, N'Harissa', N'More Places For Stuff' )
,       ( 5, N'Shichimi togarashI', N'World-wide Supply' )
,       ( 6, N'Mixed spice', N'UK Spices' )
,       ( 7, N'Old Bay Seasoning', N'US Mixed Spices' )
,       ( 8, N'Jerk spice', N'Quality Spices' );

INSERT  INTO [dbo].[Spices_New]
VALUES  ( 1, N'Advieh', N'Outlander Spices' )
,       ( 2, N'Baharat', N'Spice Source' )
,       ( 3, N'Berbere', N'International Supply' )
,       ( 4, N'Bumbu', N'Natural Farms' )
,       ( 5, N'Chaat masala', N'Sells All' )
,       ( 6, N'Chili powder', N'Jo''s Stuff' )
,       ( 9, N'Curry powder', N'VJC Processing' );

SELECT  *
FROM    [dbo].[Spices];

SELECT  *
FROM    [dbo].[Spices_New];
GO

--The actual MERGE statement script that synchronizes the dbo.Spices table with dbo.Spices_New table
MERGE [dbo].[Spices] AS [SourceTbl]
USING [dbo].[Spices_New] AS [TargetTbl]
ON ( SourceTbl.[SpiceID] = TargetTbl.[SpiceID] )
WHEN MATCHED AND SourceTbl.[SpiceMixName] <> TargetTbl.[SpiceMixName]
    OR SourceTbl.[Supplier] <> TargetTbl.[Supplier] THEN
    UPDATE SET SourceTbl.[SpiceMixName] = TargetTbl.[SpiceMixName] ,
               SourceTbl.[Supplier] = TargetTbl.[Supplier]
WHEN NOT MATCHED THEN
    INSERT ( [SpiceID] ,
             [SpiceMixName] ,
             [Supplier]
           )
    VALUES ( TargetTbl.[SpiceID] ,
             TargetTbl.[SpiceMixName] ,
             TargetTbl.[Supplier]
           )
WHEN NOT MATCHED BY SOURCE THEN
    DELETE
OUTPUT
    $action ,
    INSERTED.* ,
    DELETED.*;
GO

--Code snippet #11: TRUNCATE TABLE statement example
--The following truncate table deletes all rows from dbo.CustomProducts table
TRUNCATE TABLE [dbo].[CustomProducts];

--Code snippet #12: SELECT...INTO statement example
--The following SELECT...INTO statement creates a backup copy of Production.Product table
SELECT  *
INTO    [Production].[Product_Backup]
FROM    [Production].[Product];
GO
