--// 2550EN_05_Code_File.sql: Chapter 5 Code //-- 

USE [AdventureWorks2012];
GO

--Code snippet #1: Local variable example 
--Example #1: Declaring and assigning a value to it using the SET keyword
DECLARE @var1 [int],
        @var2 [varchar](10);

SET @var1 = 10
SET @var2 = N'MyValue1';

--Example #2: Decalring variable and assigning a value to it using the SELECT statement
DECLARE @var3 [int],
        @var4 [varchar](10);

SELECT  @var3 = 20 ,
        @var4 = N'MyValue2';

--Example #3: Assigning a value to the local variable at declaration
DECLARE @var5 [int] = 30,
        @var6 [varchar](10) = N'MyValue3';
GO

--Code snippet #2: Cursor variable example
--The statement will declare a cursor variable
DECLARE @cur_variable1 CURSOR;
GO

--Code snippet #3: Table variable example
--Example #1: The statement will declare a table variable with two columns
DECLARE @Table1 TABLE
    (
      COL1 [int],
      COL2 [varchar](30)
    );

--Example #2: The following is a basic example of inline index creation on table variable
DECLARE @Table2 TABLE (
COL1 [int],
COL2 [varchar](30),
COL3 [datetime],
	INDEX [ixc_col3] CLUSTERED (col3) 
		WITH (FILLFACTOR=80),
	INDEX [ixnc_col1_col2] NONCLUSTERED (col1, col2) 
		WITH (FILLFACTOR=80)
);
GO

--Code snippet #4: BEGIN...END code block example
BEGIN
    DECLARE @Today [datetime];
    SET @Today = CURRENT_TIMESTAMP;

    SELECT TOP 100
            *
    FROM    [HumanResources].[vEmployee];

    SELECT  @Today;
END;
GO

--Code snippet #5: IF...ELSE block example
IF ( SELECT DATENAME(dw, CURRENT_TIMESTAMP)
   ) IN ( N'Friday', N'Saturday', N'Sunday' )
    BEGIN
        SELECT  'Hey, its weekend!!!';
    END;
ELSE
    BEGIN
        SELECT  N'Its weekday!!!';
    END;
GO

--Code snippet #6: CASE statement example
SELECT CASE (SELECT DATENAME(dw, CURRENT_TIMESTAMP))
        WHEN N'Friday' THEN N'Hey, Its Friday!!!'
        WHEN N'Saturday' THEN N'Hey, its Saturday!!!'
        WHEN N'Sunday' THEN N'Hey, its Sunday!!!'
        ELSE N'Its weekday!!!'
       END;
GO

--Code snippet #7: WHILE, CONTINUE, and BREAK statement example
DECLARE @counter [int] = 0
WHILE ( @counter < 10 )
    BEGIN
        IF ( @counter < 5 )
            BEGIN
                SELECT  @counter;
                SET @counter = @counter + 1;
                CONTINUE;
            END;
        ELSE
            SET @counter = @counter + 1;
        IF @counter = 7
            BEGIN
                SELECT  @counter;
                BREAK;
            END;
    END;
GO

--Code snippet #8: Creating a view using CREATE VIEW T-SQL statement
CREATE VIEW [HumanResources].[vEmployeesWithinCompanyGroup]
AS
    SELECT  [Executive General and Administration] ,
            [Inventory Management] ,
            [Manufacturing] ,
            [Quality Assurance] ,
            [Research and Development] ,
            [Sales and Marketing]
    FROM    ( SELECT    p.BusinessEntityID ,
                        d.[GroupName]
              FROM      [HumanResources].[Employee] e
                        INNER JOIN [HumanResources].[EmployeeDepartmentHistory] dhist ON e.[BusinessEntityID] = dhist.[BusinessEntityID]
                                                              AND dhist.[EndDate] IS NULL
                        INNER JOIN [Person].[Person] p ON p.[BusinessEntityID] = e.[BusinessEntityID]
                        INNER JOIN [HumanResources].[Department] d ON dhist.[DepartmentID] = d.[DepartmentID]
            ) AS a PIVOT
( COUNT(a.[BusinessEntityID]) FOR [GroupName] IN ( [Executive General and Administration],
                                                   [Inventory Management],
                                                   [Manufacturing],
                                                   [Quality Assurance],
                                                   [Research and Development],
                                                   [Sales and Marketing] ) ) AS b
GO

--Code snippet #9: Modifying a view using ALTER VIEW T-SQL statement
ALTER VIEW [HumanResources].[vEmployeesWithinCompanyGroup]
AS
    SELECT  [Executive General and Administration] ,
            [Inventory Management] ,
            [Manufacturing] ,
            [Quality Assurance] ,
            [Research and Development] ,
            [Sales and Marketing]
    FROM    ( SELECT    p.BusinessEntityID ,
                        d.[GroupName]
              FROM      [HumanResources].[Employee] e
                        INNER JOIN [HumanResources].[EmployeeDepartmentHistory] dhist ON e.[BusinessEntityID] = dhist.[BusinessEntityID]
                                                              AND dhist.[EndDate] IS NULL
                        INNER JOIN [Person].[Person] p ON p.[BusinessEntityID] = e.[BusinessEntityID]
                        INNER JOIN [HumanResources].[Department] d ON dhist.[DepartmentID] = d.[DepartmentID]
            ) AS a PIVOT
( COUNT(a.[BusinessEntityID]) FOR [GroupName] IN ( [Executive General and Administration],
                                                   [Inventory Management],
                                                   [Manufacturing],
                                                   [Quality Assurance],
                                                   [Research and Development],
                                                   [Sales and Marketing] ) ) AS b
GO

--Code snippet #9: Deleting a view using DROP VIEW T-SQL statement
DROP VIEW [HumanResources].[vEmployeesWithinCompanyGroup]; 
GO

--Code snippet #10: Indexed view example
--Step #1: To create indexed view, first create the view:
CREATE VIEW [Sales].[vIndividualEmployeeIndexedViews]
WITH SCHEMABINDING
AS
    SELECT  p.[Title] ,
            p.[FirstName] + SPACE(1) + p.[MiddleName] + SPACE(1)
            + p.[LastName] AS [FullName] ,
            e.[JobTitle] ,
            d.[Name] AS [Department] ,
            d.[GroupName] ,
            dhist.[StartDate]
    FROM    [HumanResources].[Employee] e
            INNER JOIN [HumanResources].[EmployeeDepartmentHistory] dhist ON e.[BusinessEntityID] = dhist.[BusinessEntityID]
                                                              AND dhist.[EndDate] IS NULL
            INNER JOIN [Person].[Person] p ON p.[BusinessEntityID] = e.[BusinessEntityID]
            INNER JOIN [HumanResources].[Department] d ON dhist.[DepartmentID] = d.[DepartmentID];
GO

--Step #2: Then, create a clustered index on the view to make it indexed view
CREATE UNIQUE CLUSTERED INDEX ixc_EmployeeList ON [Sales].[vIndividualEmployeeIndexedViews]
([FullName], [JobTitle], [StartDate]);
GO

--Code snippet #11: Creating a stored procedure with CREATE PROCEDURE T-SQL statement
--Example #1: Creating a standard stored procedure
CREATE PROCEDURE [HumanResources].[uspUpdateEmployeeInfo]
    @BusinessEntityID [int] ,
    @NationalIDNumber [nvarchar](15) ,
    @BirthDate [datetime] ,
    @MaritalStatus [nchar](1) ,
    @Gender [nchar](1)
    WITH EXECUTE AS CALLER
AS
    BEGIN
        SET NOCOUNT ON;

        BEGIN TRY
            UPDATE  [HumanResources].[Employee]
            SET     [NationalIDNumber] = @NationalIDNumber ,
                    [BirthDate] = @BirthDate ,
                    [MaritalStatus] = @MaritalStatus ,
                    [Gender] = @Gender
            WHERE   [BusinessEntityID] = @BusinessEntityID;
        END TRY
        BEGIN CATCH
            EXECUTE [dbo].[uspLogError];
        END CATCH;
    END;
GO

--Example #2: Creating a natively compiled stored procedure
--Run the following code to add memory-optimized filegroup within AdventureWorks2012 database
USE [master];
GO

ALTER DATABASE [AdventureWorks2012]
ADD FILEGROUP [AW_MEMORYOPTIMIZED]
CONTAINS MEMORY_OPTIMIZED_DATA;

ALTER DATABASE [AdventureWorks2012]
ADD FILE (NAME='AdventureWorks2012_MemoryOptimized', 
FILENAME='C:\SQLData\AdventureWorks2012_MO.ndf') 
TO FILEGROUP [AW_MemoryOptimized];
GO

--Run the following code to create memory-optimized table
USE [AdventureWorks2012];
GO

CREATE TABLE Sales.SalesOrderDetail_MO (
	[SalesOrderID] [int] NOT NULL
	,[SalesOrderDetailID] [int] NOT NULL
	,[CarrierTrackingNumber] [nvarchar](25) NULL
	,[OrderQty] [smallint] NOT NULL
	,[ProductID] [int] NOT NULL
	,[SpecialOfferID] [int] NOT NULL
	,[UnitPrice] [money] NOT NULL
,[UnitPriceDiscount] [money] NOT NULL 
	,[LineTotal] [money]
,[rowguid] [uniqueidentifier] NOT NULL
,[ModifiedDate] [datetime]
,CONSTRAINT [PK_SalesOrderDetail_SalesOrderID_SalesOrderDetailID2] PRIMARY KEY NONCLUSTERED HASH (
		 [SalesOrderID]
		,[SalesOrderDetailID]) WITH (BUCKET_COUNT = 20000))
	WITH (MEMORY_OPTIMIZED = ON
		 ,DURABILITY = SCHEMA_AND_DATA);
GO

--Copy data into memory-optimized table
INSERT  INTO [Sales].[SalesOrderDetail_MO]
        SELECT  [SalesOrderID] ,
                [SalesOrderDetailID] ,
                [CarrierTrackingNumber] ,
                [OrderQty] ,
                [ProductID] ,
                [SpecialOfferID] ,
                [UnitPrice] ,
                [UnitPriceDiscount] ,
                [LineTotal] ,
                [rowguid] ,
                [ModifiedDate]
        FROM    [Sales].[SalesOrderDetail];
GO

--Create memory-optimized table to retrieve order detail information.
CREATE PROCEDURE RetriveOrderDetail
    @SalesOrderID [int]
    WITH NATIVE_COMPILATION
		,SCHEMABINDING
		,EXECUTE AS OWNER
AS
    BEGIN
		ATOMIC
		WITH (TRANSACTION ISOLATION LEVEL = SNAPSHOT
			 ,LANGUAGE = 'English')
        SELECT  [SalesOrderID] ,
                [SalesOrderDetailID] ,
                [CarrierTrackingNumber] ,
                [OrderQty] ,
                [ProductID] ,
                [SpecialOfferID] ,
                [UnitPrice] ,
                [UnitPriceDiscount] ,
                [LineTotal] ,
                [rowguid] ,
                [ModifiedDate]
        FROM    [Sales].[SalesOrderDetail_MO]
        WHERE   [SalesOrderID] = @SalesOrderID;
    END;
GO

--Code snippet #12: Modifying a stored procedure with ALTER PROCEDURE T-SQL statement
ALTER PROCEDURE [HumanResources].[uspUpdateEmployeeInfo]
    @BusinessEntityID [int] ,
    @NationalIDNumber [nvarchar](15) ,
    @BirthDate [datetime] ,
    @MaritalStatus [nchar](1) ,
    @Gender [nchar](1)
    WITH EXECUTE AS CALLER
AS
    BEGIN
        SET NOCOUNT ON;

        BEGIN TRY
            UPDATE  [HumanResources].[Employee]
            SET     [NationalIDNumber] = @NationalIDNumber ,
                    [BirthDate] = @BirthDate ,
                    [MaritalStatus] = @MaritalStatus ,
                    [Gender] = @Gender
            WHERE   [BusinessEntityID] = @BusinessEntityID;
        END TRY
        BEGIN CATCH
            EXECUTE [dbo].[uspLogError];
        END CATCH;
    END;
GO

--Code snippet #13: Deleting a stored procedure with DROP PROCEDURE T-SQL statement
DROP PROCEDURE [HumanResources].[uspUpdateEmployeeInfo];
GO

--Code snippet #14: Executing a stored procedure with EXECUTE keyword
DECLARE @RC [int]
DECLARE @StartProductID [int] = 20
DECLARE @CheckDate [datetime] = CURRENT_TIMESTAMP - 40

EXECUTE @RC = [dbo].[uspGetWhereUsedProductID] @StartProductID, @CheckDate;
GO

--Code snippet #15: Creating a function with CREATE FUNCTION T-SQL statement
--Example #1: Creating a user-defined scalar function
CREATE FUNCTION dbo.fnIsWeekday ( @p_date [datetime] )
RETURNS [bit]
AS
    BEGIN
        DECLARE @weekday [bit]
        IF (SELECT DATENAME(dw, @p_date)) IN (N'Friday', N'Saturday', N'Sunday')
            BEGIN
                SET @weekday = 0;
            END
        ELSE
            BEGIN
                SET @weekday = 1;
            END
        RETURN (@weekday)
    END;
GO

--Using a user-define scalar function
SELECT dbo.fnIsWeekday ('June 23, 2014');
GO

--Example #2: Creating a user-defined inline table-valued function
CREATE FUNCTION [dbo].[GetEmployeeDetails] ( @p_employeeid [int] )
RETURNS TABLE
AS
RETURN
    ( 
WITH    [CTE_EmployeeInfo]
          AS ( SELECT   e.[BusinessEntityID] ,
                        p.[Title] ,
                        p.[FirstName] ,
                        p.[MiddleName] ,
                        p.[LastName] ,
                        p.[Suffix] ,
                        e.[JobTitle] ,
                        pp.[PhoneNumber] ,
                        pnt.[Name] AS [PhoneNumberType] ,
                        ea.[EmailAddress] ,
                        p.[EmailPromotion] ,
                        a.[AddressLine1] ,
                        a.[AddressLine2] ,
                        a.[City] ,
                        sp.[Name] AS [StateProvinceName] ,
                        a.[PostalCode] ,
                        cr.[Name] AS [CountryRegionName]
               FROM     [HumanResources].[Employee] e
                        INNER JOIN [Person].[Person] p ON p.[BusinessEntityID] = e.[BusinessEntityID]
                        INNER JOIN [Person].[BusinessEntityAddress] bea ON bea.[BusinessEntityID] = e.[BusinessEntityID]
                        INNER JOIN [Person].[Address] a ON a.[AddressID] = bea.[AddressID]
                        INNER JOIN [Person].[StateProvince] sp ON sp.[StateProvinceID] = a.[StateProvinceID]
                        INNER JOIN [Person].[CountryRegion] cr ON cr.[CountryRegionCode] = sp.[CountryRegionCode]
                        LEFT OUTER JOIN [Person].[PersonPhone] pp ON pp.BusinessEntityID = p.[BusinessEntityID]
                        LEFT OUTER JOIN [Person].[PhoneNumberType] pnt ON pp.[PhoneNumberTypeID] = pnt.[PhoneNumberTypeID]
                        LEFT OUTER JOIN [Person].[EmailAddress] ea ON p.[BusinessEntityID] = ea.[BusinessEntityID]
             )
    SELECT  *
    FROM    [CTE_EmployeeInfo]
    WHERE   [BusinessEntityID] = @p_employeeid
)
GO

--Example #3: Creating a user-defined multistatement table-valued function
CREATE FUNCTION [dbo].[ufnRetrieveContactInformation] ( @p_contactid [int] )
RETURNS @ContactDetails TABLE
    (
      -- Columns returned by the function
      [ContactID] [int] PRIMARY KEY
                        NOT NULL ,
      [Title] [nvarchar](8) NULL ,
      [FirstName] [nvarchar](50) NULL ,
      [MiddleName] [nvarchar](50) NULL ,
      [LastName] [nvarchar](50) NULL ,
      [JobTitle] [nvarchar](50) NULL ,
      [ContactType] [nvarchar](50) NULL
    )
AS
    BEGIN
        IF @p_contactid IS NOT NULL
            BEGIN
                INSERT  INTO @ContactDetails
                        ( [ContactID] ,
                          [Title] ,
                          [FirstName] ,
                          [MiddleName] ,
                          [LastName] ,
                          [ContactType]
                        )
                        SELECT  [BusinessEntityID] ,
                                [Title] ,
                                [FirstName] ,
                                [MiddleName] ,
                                [LastName] ,
                                CASE [PersonType]
                                  WHEN N'EM' THEN N'Employee'
                                  WHEN N'SP' THEN N'Employee'
                                  WHEN N'VC' THEN N'Vendor Contact'
                                  WHEN N'SC' THEN N'Store Contact'
                                  WHEN N'IN' THEN N'Consumer'
                                  ELSE N'General Contact'
                                END
                        FROM    [Person].[Person]
                        WHERE   [BusinessEntityID] = @p_contactid;

                IF EXISTS ( SELECT  *
                            FROM    [Person].[Person] p
                            WHERE   p.[BusinessEntityID] = @p_contactid
                                    AND p.[PersonType] IN ( N'EM', N'SP' ) )
                    BEGIN
                        UPDATE  @ContactDetails
                        SET     [JobTitle] = ( SELECT   e.[JobTitle]
                                               FROM     [Person].[Person] p
                                                        INNER JOIN [HumanResources].[Employee] e ON e.[BusinessEntityID] = p.[BusinessEntityID]
                                                              AND p.[BusinessEntityID] = @p_contactid
                                             );
                    END;

                IF EXISTS ( SELECT  *
                            FROM    [Person].[Person] p
                            WHERE   p.[BusinessEntityID] = @p_contactid
                                    AND p.[PersonType] = N'VC' )
                    BEGIN
                        UPDATE  @ContactDetails
                        SET     [JobTitle] = ( SELECT   ct.[Name]
                                               FROM     [Person].[Person] p
                                                        INNER JOIN [Person].[BusinessEntityContact] bec ON bec.[PersonID] = p.[BusinessEntityID]
                                                              AND p.[BusinessEntityID] = @p_contactid
                                                        INNER JOIN [Person].[ContactType] ct ON ct.[ContactTypeID] = bec.[ContactTypeID]
                                             );                                             
                    END;

                IF EXISTS ( SELECT  *
                            FROM    [Person].[Person] p
                            WHERE   p.[BusinessEntityID] = @p_contactid
                                    AND p.[PersonType] = N'SC' )
                    BEGIN
                        UPDATE  @ContactDetails
                        SET     [JobTitle] = ( SELECT   ct.[Name]
                                               FROM     [Person].[Person] p
                                                        INNER JOIN [Person].[BusinessEntityContact] bec ON bec.[PersonID] = p.[BusinessEntityID]
                                                              AND p.[BusinessEntityID] = @p_contactid
                                                        INNER JOIN [Person].[ContactType] ct ON ct.[ContactTypeID] = bec.[ContactTypeID]
                                                        INNER JOIN [Sales].[Store] s ON bec.[BusinessEntityID] = s.[BusinessEntityID]
                                             );                                             
                    END;

                IF EXISTS ( SELECT  *
                            FROM    [Person].[Person] p
                            WHERE   p.[BusinessEntityID] = @p_contactid
                                    AND p.[PersonType] = N'IN' )
                    BEGIN
                        UPDATE  @ContactDetails
                        SET     [JobTitle] = ( SELECT   NULL
                                               FROM     [Person].[Person] p
                                                        INNER JOIN [Sales].[Customer] c ON c.[PersonID] = p.[BusinessEntityID]
                                                              AND p.[BusinessEntityID] = @p_contactid
                                                              AND c.[StoreID] IS NULL
                                             );                                             
                    END;
            END;
        RETURN;
    END;
GO

--Using a user-defined inline and multistatement table-value functions
SELECT  *
FROM    [dbo].[GetEmployeeDetails](92);
GO

SELECT  *
FROM    [dbo].[ufnRetrieveContactInformation](89);
GO

--Code snippet #16: Deleting a function with DROP FUNCTION T-SQL statement
DROP FUNCTION [dbo].[ufnRetrieveContactInformation];
GO

--Code snippet #17: Enabling nested triggers feature
USE [master];
GO

EXEC sp_configure N'show advanced options', 1;
GO
-- To update the currently configured value for advanced options.
RECONFIGURE;
GO

-- To enable the nested trigger feature.
EXEC sp_configure N'nested triggers', 1;
GO

-- To update the currently configured value for this feature.
RECONFIGURE;
GO

--Code snippet #18: Creating a DML trigger with CREATE TRIGGER T-SQL statement
CREATE TRIGGER [Production].[WorkOrder_after_trigger] ON [Production].[WorkOrder]
    AFTER INSERT
AS
    BEGIN
        DECLARE @Count INT;

        SET @Count = @@ROWCOUNT;
        IF @Count = 0
            RETURN;

        SET NOCOUNT ON;
        INSERT  INTO [Production].[TransactionHistory]
                ( [ProductID] ,
                  [ReferenceOrderID] ,
                  [TransactionType] ,
                  [TransactionDate] ,
                  [Quantity] ,
                  [ActualCost]
                )
                SELECT  inserted.[ProductID] ,
                        inserted.[WorkOrderID] ,
                        'W' ,
                        GETDATE() ,
                        inserted.[OrderQty] ,
                        0
                FROM    inserted;
    END;
GO

--Code snippet #19: Creating a DDL trigger with CREATE TRIGGER T-SQL statement
CREATE TRIGGER trig_preventDDLOOH ON DATABASE
    FOR DDL_DATABASE_LEVEL_EVENTS
AS
    BEGIN
        IF ( DATEPART(HOUR, CURRENT_TIMESTAMP) < 8
             OR DATEPART(HOUR, CURRENT_TIMESTAMP) > 17
           )
            PRINT N'You cannot perform DDL outside of normal business hours';
        ROLLBACK;
    END
GO

--Code snippet #20: TRY...CATCH block examples
--Example #1: TRYCATCH example
BEGIN TRY
    SELECT  50 / 0
END TRY
BEGIN CATCH
    SELECT  @@ERROR AS [@@ERROR]
    SELECT  ERROR_NUMBER() AS [ERROR_NUMBER] ,
            ERROR_MESSAGE() AS [ERROR_MESSAGE]
END CATCH
GO

--Example #2: TRYCATCH with THROW example
BEGIN TRY
    SELECT  50 / 0
END TRY
BEGIN CATCH
    THROW;
END CATCH
GO

--Example #3: TRYCATCH with RAISERROR example
BEGIN TRY
    SELECT  50 / 0
END TRY
BEGIN CATCH
    RAISERROR (N'Oops, a divide-by-zero error occurred.', 16, 1) WITH NOWAIT
END CATCH
GO